@@ -1,4 +1,5 @@
 import inspect
+from .smarterStore import SmarterStore
 from abc import ABCMeta, abstractmethod
 from typing import Callable, Optional, Any
 
@@ -8,7 +9,8 @@ class SmarterMessage(dict):
 
 
 class SmarterSender:
-    def __init__(self, delegate: Callable[[SmarterMessage, str], Optional[SmarterMessage]]):
+    def __init__(self, store_dir: str, delegate: Callable[[SmarterMessage, str], Optional[SmarterMessage]]):
+        self.smarter_store = SmarterStore(store_dir=store_dir)
         self.delegate = delegate
 
     def send_message(self, message: SmarterMessage, port: str) -> Optional[SmarterMessage]:
@@ -28,6 +30,7 @@ class SmarterSender:
                         Example a chart expects a table-like format, while a textField expects some text.
         :return: None
         """
+        self.smarter_store.set_pattern_data(pattern=pattern, data=data)
         message = SmarterMessage({"action": "setData",
                                   "args": {"pattern": pattern, "data": data}})
         self.send_message(message=message, port='#gui')
@@ -38,6 +41,7 @@ class SmarterSender:
         :param pattern: The front-end GUI pattern to set the data to
         :return: None
         """
+        self.smarter_store.set_pattern_data(pattern=pattern, data=None)
         message = SmarterMessage({"action": "setData",
                                   "args": {"pattern": pattern, "data": None}})
         self.send_message(message=message, port='#gui')
@@ -51,6 +55,7 @@ class SmarterSender:
             the latest added elements
         :return: None
         """
+        self.smarter_store.append_pattern_data(pattern=pattern, data=data)
         message = SmarterMessage({"action": "setData",
                                   "args": {"pattern": pattern, "data": data},
                                   "options": {"append": True, "limitLength": limit}})
@@ -65,11 +70,15 @@ class SmarterSender:
             the latest added elements
         :return: None
         """
+        self.smarter_store.prepend_pattern_data(pattern=pattern, data=data)
         message = SmarterMessage({"action": "setData",
                                   "args": {"pattern": pattern, "data": data},
                                   "options": {"prepend": True, "limitLength": limit}})
         self.send_message(message=message, port='#gui')
 
+    def get_data(self, pattern) -> SmarterMessage:
+        return self.smarter_store.get_pattern_data(pattern)
+
     def reply_back(self, message: SmarterMessage) -> None:
         """
         Takes in a message to reply back to front-end REST/Websocket topics. An equivalent  to 'return' with a message.
@@ -94,10 +103,8 @@ class _Smarter_SignatureCheckerMeta(ABCMeta):
                 )
                 if smarter_signature != flex_signature:
                     errors.append(
-                        "Abstract method {0} not implemented with "
-                        "correct signature in {1}. Expected {2}.".format(
-                            func_name, cls.__name__, smarter_signature
-                        )
+                        f"Abstract method {func_name} "
+                        f"not implemented with correct signature in {cls.__name__}. Expected {smarter_signature}."
                     )
         if errors:
             raise TypeError("\n".join(errors))
